@@ -1,6 +1,6 @@
-`twsConnect` <-
+twsConnect <-
 function (clientId=1, host='localhost', port = 7496, verbose=TRUE,
-          timeout=5, filename=NULL)
+          timeout=5, filename=NULL, blocking=FALSE)
  {
    if(is.null(getOption('digits.secs'))) 
      options(digits.secs=6)
@@ -11,14 +11,14 @@ function (clientId=1, host='localhost', port = 7496, verbose=TRUE,
    if(is.null(filename)) {
      start.time <- Sys.time()
      s <- socketConnection(host = host, port = port,
-                           open='ab', blocking=TRUE)
+                           open='ab', blocking=blocking)
 
      if(!isOpen(s)) { 
        close(s)
        stop(paste("couldn't connect to TWS on port",port))
      }
 
-     CLIENT_VERSION <- "37"
+     CLIENT_VERSION <- "38"
 
      writeBin(CLIENT_VERSION, s)
      writeBin(as.character(clientId), s)
@@ -27,7 +27,7 @@ function (clientId=1, host='localhost', port = 7496, verbose=TRUE,
      while(waiting) {
        curMsg <- readBin(s, character(), 1)
        if(length(curMsg) > 0) {
-         if(curMsg %in% as.character(39:43)) {
+         if(curMsg %in% as.character(39:44)) {
            SERVER_VERSION <- curMsg
            CONNECTION_TIME <- readBin(s,character(),1)
          }
@@ -59,7 +59,8 @@ function (clientId=1, host='localhost', port = 7496, verbose=TRUE,
     tmp <- tempfile()
     fh <- file(tmp, open='ab')
 
-    writeBin(c(as.character(length(dat)),dat), fh)
+    #writeBin(c(as.character(length(dat)),dat), fh)
+    writeBin(dat, fh)
     #for(i in dat) writeBin(i, fh)
 
     close(fh)
@@ -73,3 +74,22 @@ function (clientId=1, host='localhost', port = 7496, verbose=TRUE,
 
   }
 }
+
+is.twsConnection <- function(x)
+{
+  inherits(x, "twsConnection")
+}
+
+is.twsPlayback <- function(x)
+{
+  inherits(x, "twsPlayback")
+}
+
+isConnected <- function(x)
+{
+  if(is.twsConnection(x)) {
+    if(inherits(try(isOpen(x[[1]]), silent=TRUE), 'try-error')) {
+      FALSE
+    } else TRUE 
+  } else isOpen(x)
+} 
